//
// This class uses StackMachine.java as a client.
// Please check the comments in StackMachine.java for more information.
//

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class StackMachineGUI extends JFrame implements ComponentListener
{
    StackMachineGUI()
    {
        super ("Stack Machine Example");

        _stackMachine = new StackMachine();
        _stackSizeLabel = new JLabel("Number of elements: 0");

        Container content_pane = getContentPane();
        content_pane.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

        JPanel stack_panel = createStackPanel();

        gbc.fill    = GridBagConstraints.HORIZONTAL;
        gbc.anchor  = GridBagConstraints.CENTER;
        gbc.gridx   = 0;
        gbc.gridy   = 0;
        gbc.weightx = 1;
        gbc.gridwidth = 2;
        gbc.insets.top = 5;
        gbc.insets.left = 5;
        content_pane.add(_stackSizeLabel, gbc);

        gbc.fill      = GridBagConstraints.BOTH;
        gbc.gridy     = 1;
        gbc.gridwidth = 1;
        gbc.weighty   = 1;
        gbc.insets    = new Insets(0, 5, 5, 5);
        content_pane.add(createStackPanel(), gbc);

        gbc.gridx   = 1;
        gbc.weightx = 0;
        content_pane.add(createInputPanel(), gbc);

        setSize(400, 450);
        _stackList.adjustScroll();

        addComponentListener(this);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    public void componentHidden(ComponentEvent e) {}
    public void componentMoved(ComponentEvent e) {}
    public void componentResized(ComponentEvent e) {
        _stackList.adjustScroll ();
    }
    public void componentShown(ComponentEvent e) {}
    

    private JPanel createStackPanel ()
    {
        _stackList  = new StackList();
        _scrollPane = new JScrollPane(_stackList);
        _stackSizeLabel.setForeground(Color.black);

        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

        gbc.fill    = GridBagConstraints.BOTH;
        gbc.anchor  = GridBagConstraints.CENTER;
        gbc.gridx   = 0;
        gbc.gridy   = GridBagConstraints.RELATIVE;
        gbc.weightx = 1;
 
        gbc.weighty = 1;
        panel.add(_scrollPane, gbc);

        gbc.weighty = 0;
        panel.add(createStackButtonPanel(), gbc);

        return panel;
    }

    private JPanel createStackButtonPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridx = GridBagConstraints.RELATIVE;
        gbc.gridy = 0;
        gbc.weightx = 1;
        gbc.weighty = 1;
        panel.add(new PushButton(), gbc);
        panel.add(new PopButton(), gbc);

        return panel;
    }

    private JPanel createInputPanel()
    { 
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill    = GridBagConstraints.HORIZONTAL;
        gbc.anchor  = GridBagConstraints.NORTHWEST;
        gbc.gridx   = 0;
        gbc.gridy   = GridBagConstraints.RELATIVE;
        gbc.weightx = 1;
        panel.add(createInputFieldsPanel(), gbc);

        gbc.insets.top = 5;
        panel.add(createOperationButtonsPanel(), gbc);

        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        panel.add(new JPanel(), gbc);

        return panel;
    }

    private JPanel createOperationButtonsPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridx = GridBagConstraints.RELATIVE;
        gbc.gridy = 0;
        panel.add(new AddButton(), gbc);
        panel.add(new SubtractButton(), gbc);
        panel.add(new MultiplyButton(), gbc);
        panel.add(new DivideButton(), gbc);

        return panel;
    }

    private JPanel createInputFieldsPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill    = GridBagConstraints.HORIZONTAL;
        gbc.anchor  = GridBagConstraints.NORTHWEST;
        gbc.gridx   = 0;
        gbc.gridy   = GridBagConstraints.RELATIVE;
        gbc.weightx = 1;
        panel.add(createPushPanel(), gbc);

        return panel;
    }

    private JPanel createPushPanel()
    {
        _pushTextField = new PushTextField();

        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill   = GridBagConstraints.NONE;
        gbc.gridx  = 0;
        gbc.gridy  = GridBagConstraints.RELATIVE;
        JLabel input = new JLabel("Input:");
        input.setForeground(Color.black);
        panel.add(input, gbc);

        gbc. fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.insets.right = 0;
        panel.add(_pushTextField, gbc);

        return panel;
    }

    private class PushTextField extends JTextField implements ActionListener
    {
        PushTextField()
        {
            super (3);
            addActionListener(this);
        }

        public void actionPerformed(ActionEvent event) {
            _stackList.push (getText ());
            setText ("");
        }
    }

    private JPanel createPopPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill   = GridBagConstraints.NONE;
        gbc.gridx  = GridBagConstraints.RELATIVE;
        gbc.gridy  = 0;
        gbc.insets.right = 5;
        panel.add(new PopButton(), gbc);

        return panel;
    }

    private class PushButton extends ImageButton implements ActionListener
    {
        PushButton()
        {
            super ("images/push_button.gif", "images/push_buttonr.gif", "images/push_buttonp.gif");
            addActionListener(this);
        }

        public void actionPerformed(ActionEvent event) {
            _stackList.push (_pushTextField.getText ());
        }
    }

    private class PopButton extends ImageButton implements ActionListener
    {
        PopButton()
        {
            super ("images/pop_button.gif", "images/pop_buttonr.gif", "images/pop_buttonp.gif");
            addActionListener(this);
        }

        public void actionPerformed(ActionEvent event) {
            _stackList.pop ();
        }
    }

    private class AddButton extends ImageButton implements ActionListener
    {
        AddButton()
        {
            super ("images/add_button.gif"
                , "images/add_buttonr.gif"
                , "images/add_buttonp.gif"
            );
            addActionListener(this);
        }

        public void actionPerformed(ActionEvent event) {
            _stackList.add ();
        }
    }

    private class SubtractButton extends ImageButton implements ActionListener
    {
        SubtractButton()
        {
            super ("images/subtract_button.gif"
                , "images/subtract_buttonr.gif"
                , "images/subtract_buttonp.gif"
            );
            addActionListener(this);
        }

        public void actionPerformed(ActionEvent event) {
            _stackList.subtract ();
        }
    }

    private class MultiplyButton extends ImageButton implements ActionListener
    {
        MultiplyButton ()
        {
            super ("images/multiply_button.gif"
                , "images/multiply_buttonr.gif"
                , "images/multiply_buttonp.gif"
            );
            addActionListener(this);
        }

        public void actionPerformed (ActionEvent event) {
            _stackList.multiply ();
        }
    }

    private class DivideButton extends ImageButton implements ActionListener
    {
        DivideButton()
        {
            super ("images/divide_button.gif"
                , "images/divide_buttonr.gif"
                , "images/divide_buttonp.gif"
            );
            addActionListener(this);
        }

        public void actionPerformed(ActionEvent event) {
            _stackList.divide();
        }
    }

    private class ImageButton extends JButton
    {
        ImageButton(String button_image, String buttonr_image
            , String buttonp_image
        ) {
            super (new ImageIcon(button_image));
            setRolloverIcon(new ImageIcon(buttonr_image));
            setPressedIcon(new ImageIcon(buttonp_image));
            setFocusPainted(false);
            setBorderPainted(false);
            setContentAreaFilled(false);
            setMargin(new Insets(0, 0, 0, 0));
        }
    }

    private class StackList extends JList
    {
        StackList ()
        {
            setModel (_stackListModel = new StackMachine ());

            CustomListRenderer renderer = new CustomListRenderer();
            renderer.setBackground(Color.yellow);
            renderer.setForeground(Color.red);
            renderer.setOpaque(true);
            setCellRenderer(renderer);
        }


        void push (String value) 
        { 
            _stackListModel.push (value);
            _stackSizeLabel.setText ("Number of elements: "
                + _stackListModel.size ()
            );
            adjustScroll ();
        }

        void pop () 
        { 
            _stackListModel.pop ();
            _stackSizeLabel.setText("Number of elements: " + _stackListModel.size());
            adjustScroll ();
        }

        void add () {
            _stackListModel.add ();
            adjustScroll ();
        }
        void subtract () {
            _stackListModel.subtract ();
            adjustScroll ();
        }
        void multiply () {
            _stackListModel.multiply ();
            adjustScroll ();
        }
        void divide () {
            _stackListModel.divide ();
            adjustScroll ();
        }

        private void adjustScroll()
        {
            JViewport viewport = _scrollPane.getViewport();
            viewport.setBackground(Color.white);

            int list_size = _stackListModel.getSize();
            if (list_size == 0)
                return;

            Rectangle cell_bounds = getCellBounds(list_size - 1, list_size - 1);

            int cell_height = cell_bounds.height;
            int list_height = cell_height * list_size;
            int list_component_height = _scrollPane.getSize().height;

            if (list_component_height > list_height) {
                int position_y = -(list_component_height - list_height);
                viewport.setViewPosition(new Point(0, position_y));
            } else {
                viewport.setViewPosition(new Point(0, 0));
            }
            viewport.repaint();
        }

        private StackMachine _stackListModel;
    }

    private class CustomListRenderer extends DefaultListCellRenderer
    {
        public void paint(Graphics g)
        {
            setBackground (Color.yellow);
            super.paint(g);
            Dimension size = getSize();
            g.setColor(Color.gray);
            g.drawLine(0, 0, size.width, 0);
        }
    }

    private StackMachine  _stackMachine;
    private JLabel        _stackSizeLabel;
    private StackList     _stackList;
    private PushTextField _pushTextField;
    private JScrollPane   _scrollPane;


    //////////

    public static void main(String[] args)
    {
        StackMachineGUI frame = new StackMachineGUI();
        frame.setVisible(true);
    }
}
